<?php

namespace FiloBlu\BirthdayCoupon\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    protected $_context;
    protected $_resourceConnection;
    protected $_customerSession;
    protected $_orderFactory;
    public $subscriber_type;


    public function __construct(
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Framework\App\ResourceConnection $resourceConnection,
            \Magento\Customer\Model\Session $customerSession,
            \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderFactory
    ) {
        parent::__construct($context);
        $this->_context = $context;
        $this->_resourceConnection = $resourceConnection;
        $this->_customerSession = $customerSession;
        $this->_orderFactory = $orderFactory;
    }

    public function isModuleEnabled() {
        if($this->getConfig('filoblu_birthdaycoupon_section/options/is_enabled') == 1)
            return true;

        return false;
    }

    public function getDayOfYear($date) {

        if($date) {
            $data_array = explode('-', $date);
            $data_array[0] = 2016;
            $date = implode('-', $data_array);

            return date('z', strtotime($date));
        }

        return '';
    }

    public function getDateFromDay($day) {

        if($day != '') {
            $year = 2016;
            $offset = intval( intval( $day ) * 86400 );
            $str = date('Y-m-d', strtotime( 'Jan 1, ' . $year ) + $offset );

            return $str;
        }

        return '';
    }

    public function isLeapYear() {
        return checkdate(2,29,date('Y'));
    }

    public function getBirthdayCouponByDay($day) {

        if($day) {
            $rule_id = $this->getConfig('filoblu_birthdaycoupon_section/options/rule_id');
            if((int)$rule_id > 0) {
                $connection = $this->_resourceConnection->getConnection();
                $tableName = $this->_resourceConnection->getTableName('salesrule_coupon');
                $query = "SELECT code FROM ".$tableName." WHERE rule_id = ".$rule_id." AND code LIKE '%-".$day."' ";
                $result = $connection->fetchOne($query);
                if($result) {
                    return $result;
                }
            }
        }

        return '';
    }

    public function isBirthdayRule($rule_id) {
        if((int)$rule_id > 0) {
            $birthday_rule_id = $this->getConfig('filoblu_birthdaycoupon_section/options/rule_id');
            if($rule_id == $birthday_rule_id)
                return true;
        }

        return false;
    }

    public function isBirthdayCoupon($coupon_code) {
        $rule_id = $this->getConfig('filoblu_birthdaycoupon_section/options/rule_id');
        if($this->isModuleEnabled() && (int)$rule_id > 0 && $coupon_code != '') {
            $connection = $this->_resourceConnection->getConnection();
            $tableName = $this->_resourceConnection->getTableName('salesrule_coupon');
            $query = "SELECT code FROM ".$tableName." WHERE rule_id = ".$rule_id." AND code = '".$coupon_code."' ";
            $result = $connection->fetchOne($query);
            if($result) {
                return $result;
            }
        }

        return false;
    }

    public function getConfig($conf_path, $scope = \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT) {
        return $this->_context->getScopeConfig()->getValue($conf_path, $scope);
    }

    public function isNewsletterSubscribed($customer_email) {
        if($customer_email == '')
            return false;

        $connection = $this->_resourceConnection->getConnection();
        $tableName = $this->_resourceConnection->getTableName('newsletter_subscriber');
        $query = "SELECT birthday_coupon FROM ".$tableName." WHERE subscriber_email = '".$customer_email."' ";
        $result = $connection->fetchOne($query);
        if($result) {
            return $result;
        }

        return false;
    }

    public function checkDateRange($start_date, $end_date, $check_date) {

        if($start_date == '' || $end_date == '' || $check_date == '')
            return false;

        $start_ts = strtotime($start_date);
        $end_ts = strtotime($end_date);
        $check_ts = strtotime($check_date);

        return (($check_ts >= $start_ts) && ($check_ts <= $end_ts));
    }

    public function timeValidateCoupon($coupon_code) {
        $coupon_part = explode("-", $coupon_code);
        if(count($coupon_part) > 1) {
            $coupon_day = $coupon_part[1];

            $validate_from = $this->getDateFromDay($coupon_day);
            $data_array = explode('-', $validate_from);
            $data_array[0] = date('Y');
            $validate_from = implode('-', $data_array);

            $expired_in_days = (int)$this->getConfig('filoblu_birthdaycoupon_section/options/expire_days');
            $validate_to = date('Y-m-d', strtotime($validate_from. ' + '.$expired_in_days.' days'));
            $current_date = date('Y-m-d');

            if($this->checkDateRange($validate_from, $validate_to, $current_date))
                return true;
        }

        return false;
    }

    public function validateBirthdayCoupon($quote) {
        $validate = new \stdClass();
        $validate->result = true;

        if($quote) {
            $coupon_code = $quote->getData('coupon_code');
            if($coupon_code && $coupon_code != '') {
                $customer_email = $quote->getData('customer_email');

                if(!$subscribed_coupon = $this->isNewsletterSubscribed($customer_email)) {
                    $validate->result = "false";
                    $validate->message = "Codice coupon valido solo per gli iscritti alla newsletter.";
                }
                elseif($subscribed_coupon != $coupon_code) {
                    $validate->result = "false";
                    $validate->message = "Codice coupon non valido.";
                }
                elseif(!$this->timeValidateCoupon($coupon_code)) {
                    $validate->result = "false";
                    $validate->message = "Codice coupon scaduto.";
                }
                else {
                    $usage = $this->checkCouponUsage($quote);
                    if($usage) {
                        $validate->result = "false";
                        $validate->message = "Codice coupon già utilizzato.";
                    }
                }
            }
        }

        return $validate;
    }

    public function checkCouponUsage($quote) {
        $fromDate = date('Y').'-01-01';
        $toDate = date('Y-m-d');

        $orderCollection = $this->_orderFactory->create()
            ->addFieldToSelect('*')
            ->addFieldToFilter('customer_email', ['eq' => $quote->getData('customer_email')])
            ->addFieldToFilter('coupon_code', ['eq' => $quote->getData('coupon_code')])
            ->addAttributeToFilter('created_at', ['from'=>$fromDate, 'to'=>$toDate]);

        if((int)$orderCollection->getSize() > 0)
            return true;

        return false;
    }
}
