<?php

namespace FiloBlu\BirthdayCoupon\Model\Plugin;

class Subscriber extends \FiloBlu\ExtEbizmartsMailChimp\Model\Plugin\Subscriber {

    protected $_birthdayHelper;
    protected $_request;

    public function __construct(
        \Ebizmarts\MailChimp\Helper\Data $helper,
        \Magento\Customer\Model\ResourceModel\CustomerRepository $customer,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \FiloBlu\BirthdayCoupon\Helper\Data $birthdayHelper,
        \Magento\Framework\App\Request\Http $request
    ) {
        $this->_birthdayHelper = $birthdayHelper;
        $this->_request = $request;
        parent::__construct($helper, $customer, $customerSession, $storeManager);
    }


    public function getCouponCode($dob = null) {
        if($this->_birthdayHelper->getConfig('filoblu_birthdaycoupon_section/options/is_enabled') == 1) {
            if($dob) {
                $day_of_year = $this->_birthdayHelper->getDayOfYear($dob);
                $coupon_code = $this->_birthdayHelper->getBirthdayCouponByDay($day_of_year);
                if($coupon_code && $coupon_code != '') {
                    $this->_request->setPostValue('birthday_coupon', $coupon_code);
                }
            }
        }
    }

    public function beforeSubscribeCustomerById(
        $subscriber, $customerId
    ) {
        $customer = $this->_customer->getById($customerId);
        $storeId = $customer->getStoreId();

        if ($this->_helper->isMailChimpEnabled($storeId)) {

            $email = $customer->getEmail();
            $dob = $customer->getDob();
            $this->getCouponCode($dob);

            $subscriber->loadByEmail($email);

            $api = $this->_api;

            $status = 'subscribed';

            $mergeVars = $this->_helper->getMergeVars($subscriber, true, false);

            try {

                $md5HashEmail = md5(strtolower($email));

                if ($subscriber->getMailchimpId()) {

                    $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                } else {
                    $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    if (isset($return["id"])) {
                        $subscriber->setMailchimpId($return["id"]);
                    }
                }
            } catch (\Exception $e) {
                $this->_helper->log($e->getMessage());
            }
        }
//        }
        return array($customerId);
    }

    public function beforeSubscribe($subscriber, $email) {

        $storeId = $this->_storeManager->getStore()->getId();

        if ($this->_helper->isMailChimpEnabled($storeId)) {

            $isSubscribeOwnEmail = $this->_customerSession->isLoggedIn() && $this->_customerSession->getCustomerDataObject()->getEmail() == $email;

            if (!$this->_helper->isDoubleOptInEnabled($storeId) || $isSubscribeOwnEmail) {

                $api = $this->_api;

                $status = 'subscribed';
                $dob = $this->_request->getParam('dob-newsletter-value');
                $this->getCouponCode($dob);

                $mergeVars = $this->_helper->getMergeVars($subscriber, false, false);

                try {
                    $md5HashEmail = md5(strtolower($email));

                    if ($subscriber->getMailchimpId()) {
                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    } else {

                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                        if (isset($return["id"])) {
                            $subscriber->setMailchimpId($return["id"]);
                        }
                    }
                } catch (\Exception $e) {
                    $this->_helper->log($e->getMessage());
                }
            }
        }
        return array($email);
    }
}
